searchData={"items":[{"type":"module","title":"ModBoss","doc":"Human-friendly modbus reading, writing, and translation.\n\nRead and write modbus values by name, with automatic encoding and decoding.","ref":"ModBoss.html"},{"type":"function","title":"ModBoss.read/4","doc":"Read from modbus using named mappings.\n\nThis function takes either an atom or a list of atoms representing the mappings to read,\nbatches the mappings into contiguous addresses per type, then reads and decodes the values\nbefore returning them.\n\nFor each batch, `read_func` will be called with the type of register (`:holding_register`,\n`:input_register`, `:coil`, or `:discrete_input`), the starting address for the batch\nto be read, and the count of addresses to read from. It must return either `{:ok, result}`\nor `{:error, message}`.\n\nIf a single name is requested, the result will be an :ok tuple including the singule result\nfor that named mapping. If a list of names is requested, the result will be an :ok tuple\nincluding a map with mapping names as keys and mapping values as results.","ref":"ModBoss.html#read/4"},{"type":"function","title":"Opts - ModBoss.read/4","doc":"* `:decode` — if `false`, returns the \"raw\" result as provided by `read_func`; defaults to `true`","ref":"ModBoss.html#read/4-opts"},{"type":"function","title":"Examples - ModBoss.read/4","doc":"read_func = fn register_type, starting_address, count ->\n      result = custom_read_logic(…)\n      {:ok, result}\n    end\n\n    # Read one mapping\n    ModBoss.read(SchemaModule, read_func, :foo)\n    {:ok, 75}\n\n    # Read multiple mappings\n    ModBoss.read(SchemaModule, read_func, [:foo, :bar, :baz])\n    {:ok, %{foo: 75, bar: \"ABC\", baz: true}}\n\n    # Read *all* readable mappings\n    ModBoss.read(SchemaModule, read_func, :all)\n    {:ok, %{foo: 75, bar: \"ABC\", baz: true, qux: 1024}}\n\n    # Get \"raw\" Modbus values (as returned by `read_func`)\n    ModBoss.read(SchemaModule, read_func, :all, decode: false)\n    {:ok, %{foo: 75, bar: [16706, 17152], baz: 1, qux: 1024}}","ref":"ModBoss.html#read/4-examples"},{"type":"type","title":"ModBoss.read_func/0","doc":"","ref":"ModBoss.html#t:read_func/0"},{"type":"type","title":"ModBoss.register_type/0","doc":"","ref":"ModBoss.html#t:register_type/0"},{"type":"type","title":"ModBoss.values_to_write/0","doc":"","ref":"ModBoss.html#t:values_to_write/0"},{"type":"function","title":"ModBoss.write/3","doc":"Write to modbus using named mappings.\n\nModBoss automatically encodes your `values`, then batches any encoded values destined for\ncontiguous registers—creating separate batches per register type.\n\nFor each batch, `write_func` will be called with the type of register (`:holding_register` or\n`:coil`), the starting address for the batch to be written, and a list of values to write.\nIt must return either `:ok` or `{:error, message}`.\n\n> #### Batch values {: .info}\n>\n> Each batch will contain **either a list or an individual value** based on the number of\n> addresses to be written—so you should be prepared for both.\n\n> #### Non-atomic writes! {: .warning}\n>\n> While `ModBoss.write/3` has the _feel_ of being atomic, it's important to recognize that it\n> is not! It's fully possible that a write might fail after prior writes within the same call to\n> `ModBoss.write/3` have already succeeded.\n>\n> Within `ModBoss.write/3`, if any call to `write_func` returns an error tuple,\n> the function will immediately abort, and any subsequent writes will be skipped.","ref":"ModBoss.html#write/3"},{"type":"function","title":"Example - ModBoss.write/3","doc":"write_func = fn register_type, starting_address, value_or_values ->\n      result = custom_write_logic(…)\n      {:ok, result}\n    end\n\n    iex> ModBoss.write(MyDevice.Schema, write_func, foo: 75, bar: \"ABC\")\n    :ok","ref":"ModBoss.html#write/3-example"},{"type":"type","title":"ModBoss.write_func/0","doc":"","ref":"ModBoss.html#t:write_func/0"},{"type":"module","title":"ModBoss.Encoding","doc":"Built-in encoding/decoding functions to get you started.\n\nTo make use of these functions, use the `:as` option in your `ModBoss.Schema` but leave off\nthe `encode_` or `decode_` prefix.\n\nIn other words, to use the built-in ASCII translation, specifiy `as: :ascii` in your schema.","ref":"ModBoss.Encoding.html"},{"type":"module","title":"Note about that extra arg… - ModBoss.Encoding","doc":"Note that for built-in `encode_*` functions, we pass not just the value but also the mapping\nitself. That's why you'll see an extra argument passed to these encoders!\n\nWe do this in order to provide more generic and helpful functions out the box (like\nencoding of ASCII, which requires knowledge of how many registers we're encoding for). However,\nwhen providing your own `encode_*` functions, you'll only be passed the value to be encoded\n(and not the mapping).","ref":"ModBoss.Encoding.html#module-note-about-that-extra-arg"},{"type":"function","title":"ModBoss.Encoding.decode_ascii/1","doc":"Decode integer `value`(s) representing ASCII characters to text\n\n* Up to 2 ASCII characters are stored per register.\n* `0` is interpreted as a terminator.","ref":"ModBoss.Encoding.html#decode_ascii/1"},{"type":"function","title":"Examples - ModBoss.Encoding.decode_ascii/1","doc":"iex> decode_ascii(18537)\n    {:ok, \"Hi\"}\n\n    iex> decode_ascii([18537, 8448, 0])\n    {:ok, \"Hi!\"}","ref":"ModBoss.Encoding.html#decode_ascii/1-examples"},{"type":"function","title":"ModBoss.Encoding.decode_boolean/1","doc":"Interpret `1` as `true` and `0` as `false`","ref":"ModBoss.Encoding.html#decode_boolean/1"},{"type":"function","title":"ModBoss.Encoding.decode_signed_int/1","doc":"Decode `value` to a signed integer.","ref":"ModBoss.Encoding.html#decode_signed_int/1"},{"type":"function","title":"Examples - ModBoss.Encoding.decode_signed_int/1","doc":"iex> decode_signed_int(77)\n    {:ok, 77}\n\n    iex> decode_signed_int(65_535)\n    {:ok, -1}","ref":"ModBoss.Encoding.html#decode_signed_int/1-examples"},{"type":"function","title":"ModBoss.Encoding.decode_unsigned_int/1","doc":"Decode `value` to an unsigned integer.","ref":"ModBoss.Encoding.html#decode_unsigned_int/1"},{"type":"function","title":"Examples - ModBoss.Encoding.decode_unsigned_int/1","doc":"iex> decode_unsigned_int(65_535)\n    {:ok, 65_535}","ref":"ModBoss.Encoding.html#decode_unsigned_int/1-examples"},{"type":"function","title":"ModBoss.Encoding.encode_ascii/2","doc":"Encode `text` to integers representing ASCII characters.\n\n* Prepares zero or more characters to be stored in contiguous registers.\n* Up to 2 ASCII characters can be stored per register.\n* `0` is used as a terminator if fewer than the maximum characters are encoded.\n* The `ModBoss.Mapping` is required in order to determine how many characters are supported.","ref":"ModBoss.Encoding.html#encode_ascii/2"},{"type":"function","title":"Examples - ModBoss.Encoding.encode_ascii/2","doc":"iex> encode_ascii(\"Hi!\", %ModBoss.Mapping{register_count: 3})\n    {:ok, [18537, 8448, 0]}\n\n    iex> {:error, _too_many_characters} = encode_ascii(\"Hi!\", %ModBoss.Mapping{register_count: 1})","ref":"ModBoss.Encoding.html#encode_ascii/2-examples"},{"type":"function","title":"ModBoss.Encoding.encode_boolean/2","doc":"Encode `true` as `1` and `false` as `0`","ref":"ModBoss.Encoding.html#encode_boolean/2"},{"type":"function","title":"ModBoss.Encoding.encode_signed_int/2","doc":"Encode `value` as a signed integer.\n\nValid values are -32,768 to 32,767.\n\nThis function assumes the expected output is a regular Elixir integer. It simply provides a\nguard against overly large values, then returns the provided value.","ref":"ModBoss.Encoding.html#encode_signed_int/2"},{"type":"function","title":"Examples - ModBoss.Encoding.encode_signed_int/2","doc":"iex> {:ok, -32_768} = encode_signed_int(-32_768, %{})\n    iex> {:error, _too_large} = encode_signed_int(32_768, %{})","ref":"ModBoss.Encoding.html#encode_signed_int/2-examples"},{"type":"function","title":"ModBoss.Encoding.encode_unsigned_int/2","doc":"Encode `value` as an unsigned integer.\n\nValid values are 0 to 65,535.\n\nThis function assumes the expected output is a regular Elixir integer. It simply provides a\nguard against overly large values, then returns the provided value.","ref":"ModBoss.Encoding.html#encode_unsigned_int/2"},{"type":"function","title":"Examples - ModBoss.Encoding.encode_unsigned_int/2","doc":"iex> {:ok, 65_535} = encode_unsigned_int(65_535, %{})\n    iex> {:error, _too_large} = encode_unsigned_int(65_536, %{})","ref":"ModBoss.Encoding.html#encode_unsigned_int/2-examples"},{"type":"module","title":"ModBoss.Mapping","doc":"Struct representing the Modbus mapping.","ref":"ModBoss.Mapping.html"},{"type":"type","title":"ModBoss.Mapping.t/0","doc":"","ref":"ModBoss.Mapping.html#t:t/0"},{"type":"module","title":"ModBoss.Schema","doc":"Macros for establishing Modbus schema.\n\nThe schema allows names to be assigned to individual registers or groups of contiguous\nregisters along with encoder/decoder functions. It also allows registers to be flagged\nas readable and/or writable.","ref":"ModBoss.Schema.html"},{"type":"module","title":"Naming an address - ModBoss.Schema","doc":"You'll name a Modbus address with this format:\n\n    holding_register 17, :outdoor_temp, as: {ModBoss.Encoding, :signed_int}\n\nThis establishes address 17 as a holding register with the name `:outdoor_temp`.\nThe raw value from the register will be passed to `ModBoss.Encoding.decode_signed_int/1`\nbefore being returned.\n\nSimilarly, to set aside a **group of** registers to hold a single logical value,\nit would look like:\n\n    holding_register 20..23, :model_name, as: {ModBoss.Encoding, :ascii}\n\nThis establishes addresses 20–23 as holding registers with the name `:model_name`. The raw\nvalues from these registers will be passed (as a list) to `ModBoss.Encoding.decode_ascii/1`\nbefore being returned.","ref":"ModBoss.Schema.html#module-naming-an-address"},{"type":"module","title":"Mode - ModBoss.Schema","doc":"All registers are read-only by default. Use `mode: :rw` to allow both reads & writes.\nOr use `mode: :w` to mark a register as write-only.","ref":"ModBoss.Schema.html#module-mode"},{"type":"module","title":"Automatic encoding/decoding - ModBoss.Schema","doc":"Depending on whether a mapping is flagged as readable/writable, it is expected that you\nwill provide functions with `encode_` or `decode_` prepended to the value provided by the `:as`\noption.\n\nFor example, if you specify `as: :on_off` for a read-only register, ModBoss will expect that\nthe schema module defines an `encode_on_off/1` function that accepts the value to encode and\nreturns either `{:ok, encoded_value}` or `{:error, messsage}`.\n\nIf the function to be used lives outside of the current module, a tuple including the module\nname can be passed. For example, you can use built-in translation from `ModBoss.Encoding` such\nas `:boolean`, `:signed_int`, `:unsigned_int`, and `:ascii`.\n\n> #### output of `encode_*` {: .info}\n>\n> Your encode function may need to encode for **one or multiple** registers, depending on the\n> mapping. You are free to return either a single value or a list of values—the important thing\n> is that the number of values returned needs to match the number of registers for your mapping.\n> If it doesn't, ModBoss will detect that and return an error during encoding.\n>\n> For example, if encoding \"ABC!\" as ascii into a mapping with 3 registers, this would\n> technically only \"require\" 2 registers (one 16-bit register for every 2 characters).\n> However, your encoding should return a list of 3 values if that's what you've assigned\n> to the mapping in your schema.\n\n> #### input to `decode_*` {: .info}\n>\n> When decoding a single register, the decode function will be passed the single value from that\n> register as provided by your read function.\n>\n> When decoding multiple registers (e.g. in `ModBoss.Encoding.decode_ascii/1`), the decode\n> function will be passed a **List** of values.","ref":"ModBoss.Schema.html#module-automatic-encoding-decoding"},{"type":"module","title":"Example - ModBoss.Schema","doc":"defmodule MyDevice.Schema do\n      use ModBoss.Schema\n\n      modbus_schema do\n        holding_register 1..5, :model, as: {ModBoss.Encoding, :ascii}\n        holding_register 6, :outdoor_temp, as: {ModBoss.Encoding, :signed_int}\n        holding_register 7, :indoor_temp, as: {ModBoss.Encoding, :unsigned_int}\n\n        input_register 200, :foo, as: {ModBoss.Encoding, :unsigned_int}\n        coil 300, :bar, as: :on_off, mode: :rw\n        discrete_input 400, :baz, as: {ModBoss.Encoding, :boolean}\n      end\n\n      def encode_on_off(:on), do: {:ok, 1}\n      def encode_on_off(:off), do: {:ok, 0}\n\n      def decode_on_off(1), do: {:ok, :on}\n      def decode_on_off(0), do: {:ok, :off}\n    end","ref":"ModBoss.Schema.html#module-example"},{"type":"macro","title":"ModBoss.Schema.coil/3","doc":"Adds a coil to a schema.","ref":"ModBoss.Schema.html#coil/3"},{"type":"macro","title":"Opts - ModBoss.Schema.coil/3","doc":"* `:mode` — Makes the mapping readable/writable — can be one of `[:r, :rw, :w]` (default: `:r`)\n* `:as` — Determines which encoding/decoding functions to use when writing/reading values.\n  See explanation of [automatic encoding/decoding](ModBoss.Schema.html#module-automatic-encoding-decoding).","ref":"ModBoss.Schema.html#coil/3-opts"},{"type":"macro","title":"ModBoss.Schema.discrete_input/3","doc":"Adds a read-only discrete input to a schema.","ref":"ModBoss.Schema.html#discrete_input/3"},{"type":"macro","title":"Opts - ModBoss.Schema.discrete_input/3","doc":"* `:as` — Determines which decoding functions to use when reading values.\n  See explanation of [automatic encoding/decoding](ModBoss.Schema.html#module-automatic-encoding-decoding).","ref":"ModBoss.Schema.html#discrete_input/3-opts"},{"type":"macro","title":"ModBoss.Schema.holding_register/3","doc":"Adds a holding register to a schema.","ref":"ModBoss.Schema.html#holding_register/3"},{"type":"macro","title":"Opts - ModBoss.Schema.holding_register/3","doc":"* `:mode` — Makes the mapping readable/writable — can be one of `[:r, :rw, :w]` (default: `:r`)\n* `:as` — Determines which encoding/decoding functions to use when writing/reading values.\n  See explanation of [automatic encoding/decoding](ModBoss.Schema.html#module-automatic-encoding-decoding).","ref":"ModBoss.Schema.html#holding_register/3-opts"},{"type":"macro","title":"ModBoss.Schema.input_register/3","doc":"Adds a read-only input register to a schema.","ref":"ModBoss.Schema.html#input_register/3"},{"type":"macro","title":"Opts - ModBoss.Schema.input_register/3","doc":"* `:as` — Determines which decoding functions to use when reading values.\n  See explanation of [automatic encoding/decoding](ModBoss.Schema.html#module-automatic-encoding-decoding).","ref":"ModBoss.Schema.html#input_register/3-opts"},{"type":"macro","title":"ModBoss.Schema.modbus_schema/1","doc":"Establishes a Modbus schema in the current module.","ref":"ModBoss.Schema.html#modbus_schema/1"},{"type":"extras","title":"Overview","doc":"# ModBoss\n\n \n\n[![Elixir CI](https://github.com/goodpixel/modboss/actions/workflows/elixir.yml/badge.svg?branch=main)](https://github.com/goodpixel/modboss/actions/workflows/elixir.yml)","ref":"readme.html"},{"type":"extras","title":"Show that Bus who's Boss! - Overview","doc":"ModBoss is an Elixir library that maps Modbus registers to human-friendly names and provides\nautomatic encoding/decoding of values—making your application logic simpler and more readable,\nand making testing of modbus concerns easier.\n\nNote that ModBoss doesn't handle the actual reading/writing of modbus registers—it simply assists\nin providing friendlier access to register values. You'll likely be wrapping another library such\nas [Modbux](https://hexdocs.pm/modbux/readme.html) for the actual reads/writes.","ref":"readme.html#show-that-bus-who-s-boss"},{"type":"extras","title":"Installation - Overview","doc":"If [available in Hex](https://hex.pm/docs/publish), the package can be installed\nby adding `modboss` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:modboss, \"~> 0.1.0\"}\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Usage - Overview","doc":"","ref":"readme.html#usage"},{"type":"extras","title":"1. Map your schema - Overview","doc":"Starting with the type of register, you'll define the addresses to include and a friendly name\nfor the mapping.\n\nThe `:as` option dictates how values will be encoded before being written to Modbus or decoded\nafter being read from Modbus. You can use translation functions from another module—like those\nfound in `ModBoss.Encoding`—or provide your own as shown here with `as: :fw_version`.\n\nWhen providing your own translation functions, ModBoss expects that you'll provide functions\ncorresponding to the `:as` option but with `encode_` / `decode_` prefixes added as applicable.\nThese functions will receive the value to be translated and should return either\n`{:ok, translated_value}` or `{:error, message}`.\n\n```elixir\ndefmodule MyDevice.Schema do\n  use ModBoss.Schema\n\n  modbus_schema do\n    holding_register 1, :outdoor_temp, as: {ModBoss.Encoding, :signed_int}\n    holding_register 2..5, :model_name, as: {ModBoss.Encoding, :ascii}\n    holding_register 6, :version, as: :fw_version, mode: :rw\n    # Also supports: input_register / coil / discrete_input\n  end\n\n  def encode_fw_version(value) do\n    encoded_value = do_encode(value)\n    {:ok, encoded_value}\n  end\n\n  def decode_fw_version(value) do\n    decoded_value = some_decode_logic(value)\n    {:ok, decoded_value}\n  end\nend\n```\n\nIn this example:\n* **Holding register at address 1** is named `outdoor_temp` and uses the built-in `signed_int`\n  decoder that ships with ModBoss.\n* **Holding registers 2–5** are grouped under the name `model_name` and use a built-in ASCII\n  decoder.\n* **Holding register 6** is named `version` and uses `encode_fw_version/1` and `decode_fw_version/1`\n  to translate values being written or read respectively.","ref":"readme.html#1-map-your-schema"},{"type":"extras","title":"2. Provide generic read/write functions - Overview","doc":"You'll need to provide a `read_func/3` and a `write func/3` for actually\ninteracting on the Modbus. In practice, these functions will likely build on a library like\n[Modbux](https://hexdocs.pm/modbux/readme.html) along with state stored in a GenServer (e.g.\na `modbux_pid`, IP Address, etc.) to perform the read/write operations.\n\nFor each batch, the read_func will be provided the type of register\n(`:holding_register`, `:input_register`, `:coil`, or `:discrete_input`), the starting address,\nand the number of addresses to read. It must return either `{:ok, result}` or `{:error, message}`.\n\n```elixir\nread_func = fn register_type, starting_address, count ->\n  result = custom_read_logic(…)\n  {:ok, result}\nend\n```\n\nFor each batch, the `write_func` will be provided the type of register (`:holding_register` or\n`:coil`), the starting address for the batch to be written, and a list of values to write.\nIt must return either `:ok` or `{:error, message}`.\n\n```elixir\nwrite_func = fn register_type, starting_address, value_or_values ->\n  result = custom_write_logic(…)\n  {:ok, result}\nend\n```","ref":"readme.html#2-provide-generic-read-write-functions"},{"type":"extras","title":"3. Read & Write by name! - Overview","doc":"From here you can read and write by name…\n\n#### Requesting a single value returns just one value:\n```elixir\niex> ModBoss.read(MyDevice.Schema, read_func, :outdoor_temp)\n{:ok, 72}\n```\n\n#### Requesting multiple values returns a map:\n```elixir\niex> ModBoss.read(MyDevice.Schema, read_func, [:outdoor_temp, :model_name, :version])\n{:ok, %{outdoor_temp: 72, model_name: \"AI4000\", version: \"0.1\"}}\n```\n\n#### Writing is performed via a keyword list or map:\n```elixir\niex> ModBoss.write(MyDevice.Schema, write_func, version: \"0.2\")\n:ok\n```","ref":"readme.html#3-read-write-by-name"},{"type":"extras","title":"Benefits - Overview","doc":"Extracting your Modbus schema allows you to **isolate the encode/decode logic**\nmaking it much **more testable**. Your primary application logic becomes **simpler and more\nreadable** since it references registers by name and doesn't need to worry about encoding/decoding\nof values. It also becomes fairly straightforward to set up **virtual devices** with the exact\nsame register mappings as your physical devices (e.g. using an Elixir Agent to hold the state of\nthe registers in a map). And it makes for **easier troubleshooting** since you don't need to\nmemorize (or look up) the register mappings when you're at an `iex` prompt.","ref":"readme.html#benefits"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.38.1"}}